/**
 * @file
 * A Backbone Model for the edit_ui block.
 */

(function (Drupal, Backbone) {

  "use strict";

  /**
   * Backbone model for the edit_ui block.
   */
  Drupal.editUiBlock.BlockModel = Backbone.Model.extend({
    defaults: {
      plugin_id: false,
      region: false,
      weight: 0,
      label: '',
      html_id: 0,
      block: null,
      isVisible: false,
      isSelected: false,
      isDragging: false
    },

    /**
     * {@inheritdoc}
     */
    toJSON: function() {
      return {
        region: this.get('region'),
        weight: this.get('weight')
      };
    },

    /**
     * Set block visible state.
     */
    visible: function() {
      this.set({isVisible: true});
    },

    /**
     * Set block selected state.
     */
    select: function() {
      this.set({isSelected: true});
    },

    /**
     * Set block not selected state.
     */
    unselect: function() {
      this.set({isSelected: false});
    },

    /**
     * Set block element.
     *
     * @param {jQuery} $block
     *   Block element.
     */
    setBlock: function($block) {
      this.set({block: $block});
    },

    /**
     * Set block dragging state.
     *
     * @param {number} width
     *   Dragging block width.
     * @param {number} height
     *   Dragging block height.
     * @param {object} offset
     *   Dragging block offset.
     */
    startDrag: function(width, height, offset) {
      this.set({
        isDragging: true,
        width: width,
        height: height,
        offset: offset
      });
    },

    /**
     * Set block not dragging state.
     */
    stopDrag: function() {
      this.set({isDragging: false});
    },

    /**
     * Drop block in a new region.
     */
    drop: function(region) {
      var weight = 0;
      var blocks = Drupal.editUiBlock.collections.blockCollection.where({region: region});

      blocks.forEach(function(block){
        if (block.get('weight') > weight) {
          weight = block.get('weight');
        }
      });

      this.save({
        region: region,
        weight: weight + 1
      });
    }
  });

}(Drupal, Backbone));
