/**
 * @file
 * A Backbone view for the edit_ui current page.
 */

(function (Drupal, Backbone, $) {
  "use strict";

  /**
   * Backbone view for the edit_ui page.
   */
  Drupal.editUiBlock.PageVisualView = Backbone.View.extend({
    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      this.blocks = [];

      // Block collection listener.
      this.listenTo(this.collection, 'add', this.blockAdded);
      $(document).on('drupalContextualLinkAdded.editUiBlockPageVisualView', this.contextualLinkAdded.bind(this));

      // Initialize collection.
      this.collection.fetch();
    },

    /**
     * {@inheritdoc}
     */
    remove: function() {
      $(document).off('.editUiBlockPageVisualView');
      Backbone.View.prototype.remove.apply(this, arguments);
    },

    /**
     * Block added callback.
     *
     * @param {Drupal.editUiBlock.BlockModel} block
     *   Block model instance.
     */
    blockAdded: function (block) {
      var id, $el;

      if (block.get('plugin_id') == 'system_main_block') {
        return;
      }

      id = block.get('html_id');
      $el = $('#' + id);

      if ($el.find('.contextual').length > 0) {
        // Init the block view.
        this.initView($el, block);
      } else {
        // Save data for re-use after drupalContextualLinkAdded events.
        this.blocks.push({
          el: $el,
          block: block
        });
      }
    },

    /**
     * Contextual link added callback.
     *
     * @param {Event} event
     *   Event object.
     * @param {Object} data
     *   Callback data.
     */
    contextualLinkAdded: function(event, data) {
      for (var i in this.blocks) {
        if (this.blocks[i].el.get(0) == data.$region.get(0)) {
          this.initView(
            data.$region,
            this.blocks[i].block
          );
          delete this.blocks[i];
        }
      }
    },

    /**
     * Initialize a new block view.
     *
     * @param {jQuery} $el
     *   DOM element representing the block.
     * @param {jQuery} $contextual
     *   Contextual wrapper element.
     * @param {Drupal.editUiBlock.BlockModel} block
     *   Block model instance.
     */
    initView: function($el, block) {
      var view = new Drupal.editUiBlock.BlockVisualView({
        el: $el,
        model: block
      });
      Drupal.editUiBlock.views.blockVisualViews.push(view);
      $(document).trigger('drupalEditUiBlockAdded', {
        $el: $el,
        model: block
      });
    }
  });

}(Drupal, Backbone, jQuery));
