/**
 * @file
 * A Backbone view for the edit_ui region element.
 */

(function (Drupal, Backbone) {
  "use strict";

  /**
   * Backbone view for the edit_ui region.
   */
  Drupal.editUiRegion.RegionVisualView = Backbone.View.extend({
    /**
     * Custom data.
     */
    activeClass: 'active',

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      this.listenTo(this.model, 'change:isActive', this.render);
      this.listenTo(this.model, 'init', this.initData);
      this.listenTo(this.model, 'dragOut', this.dragOut);
      this.listenTo(this.model, 'dragIn', this.dragIn);
      this.listenTo(this.model, 'drop', this.drop);

      // Init model state.
      this.model.setBlock(this.$el);
    },

    /**
     * {@inheritdoc}
     */
    render: function() {
      this.$el.toggleClass(this.activeClass, this.model.get('isActive'));
    },

    /**
     * Init region data.
     */
    initData: function() {
      var width = this.$el.width();
      var height = this.$el.height();
      var offset = this.$el.offset();
      this.left = offset.left;
      this.right = offset.left + width;
      this.top = offset.top;
      this.bottom = offset.top + height;
    },

    /**
     * DragOut event callback.
     */
    dragOut: function(region, args) {
      if (args.x > this.left && args.x < this.right && args.y > this.top && args.y < this.bottom) {
        // Enter the region.
        Drupal.editUiRegion.collections.regionCollection.setActiveRegion(region);
      }
    },

    /**
     * DragIn event callback.
     */
    dragIn: function(region, args) {
      if (args.x < this.left || args.x > this.right || args.y < this.top || args.y > this.bottom) {
        // Leave the region.
        Drupal.editUiRegion.collections.regionCollection.setActiveRegion();
      } else {
        // TODO: Manage block order.
      }
    },

    /**
     * Drop event callback.
     */
    drop: function(region, args) {
      args.block.drop(this.model.get('region'));
    }
  });

}(Drupal, Backbone));
