<?php

/**
 * @file
 * Contains \Drupal\edit_ui\Controller\EditUiBlockController.
 */

namespace Drupal\edit_ui\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Component\Utility\Html;
use Drupal\Component\Serialization\Json;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Controller managing edit_ui backbone block model.
 */
class EditUiBlockController extends ControllerBase {

  /**
   * Returns all blocks for the current theme.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function listAction() {
    $rendered = array();

    $theme  = \Drupal::config('system.theme')->get('default');
    $blocks = entity_load_multiple_by_properties('block', array('theme' => $theme));
    foreach ($blocks as $block_id => $block) {
      $rendered[] = array(
        'id'        => $block_id,
        'plugin_id' => $block->getPluginId(),
        'region'    => $block->getRegion(),
        'weight'    => $block->getWeight(),
        'label'     => $block->label(),
        'html_id'   => Html::getUniqueId('block-' . $block_id),
      );
    }

    return new JsonResponse($rendered);
  }

  /**
   * Returns the requested block.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function createAction(Request $request) {
    $rendered = array();
    return new JsonResponse($rendered);
  }

  /**
   * Returns the requested block.
   *
   * @param string $block_id
   *   The block instance ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function readAction($block_id) {
    $rendered = array();
    $block    = $this->getBlock($block_id);

    $rendered = array(
      'id'        => $block_id,
      'plugin_id' => $block->getPluginId(),
      'region'    => $block->getRegion(),
      'weight'    => $block->getWeight(),
      'label'     => $block->label(),
      'html_id'   => Html::getUniqueId('block-' . $block_id),
    );

    return new JsonResponse($rendered);
  }

  /**
   * Returns the requested block.
   *
   * @param string $block_id
   *   The block instance ID.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function updateAction($block_id, Request $request) {
    $rendered = array();
    $block    = $this->getBlock($block_id);

    $data = Json::decode($request->getContent());
    if (!empty($data['region'])) {
      $block->setRegion($data['region']);
    }
    if (!empty($data['weight'])) {
      $block->setWeight($data['weight']);
    }
    $block->save();

    return new JsonResponse($rendered);
  }

  /**
   * Returns the requested block.
   *
   * @param string $block_id
   *   The block instance ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function deleteAction($block_id) {
    $rendered = array();
    $block    = $this->getBlock($block_id);

    $block->delete();

    return new JsonResponse($rendered);
  }

  /**
   * Get the requested block.
   *
   * @param string $block_id
   *   The block instance ID.
   *
   * @return \Drupal\block\Entity\Block
   *   The block entity.
   */
  public function getBlock($block_id) {
    if (empty($block_id)) {
      throw new BadRequestHttpException(t('No block id specified.'));
    }

    $block = entity_load('block', $block_id);

    if (empty($block)) {
      throw new BadRequestHttpException(t('Block not found.'));
    }

    return $block;
  }

}
